﻿using System;
using System.Collections.Generic;

namespace BabyShop.Site.Foundation
{
    public abstract class PersistentDataContainer
    {
        protected IDictionary<string, object> DataStore { get; private set; }

        public static Func<PersistentDataContainer> LoadCurrent { get; set; }

        public static PersistentDataContainer Current
        {
            get
            {
                return LoadCurrent == null ? null : LoadCurrent();
            }
        }

        protected PersistentDataContainer()
        {
            DataStore = RecoveryStore();
            if(DataStore == null) {
                DataStore = new Dictionary<string, object>();
                PersistentStore(DataStore);
            }
        }

        protected abstract IDictionary<string, object> RecoveryStore();

        protected abstract void PersistentStore(IDictionary<string, object> store);

        public virtual T Read<T>(string keyName)
        {
            if (DataStore.Keys.Contains(keyName))
                return (T) DataStore[keyName];
            return default(T);
        }

        public virtual T Read<T>()
        {
            return Read<T>(typeof(T).ToString());
        }

        public virtual void Add<T>(string key, T value)
        {
            DataStore[key] = value;
        }

        public virtual void Add<T>(T value)
        {
            DataStore[typeof(T).ToString()] = value;
        }

        public virtual void Remove<T>(string keyName)
        {
            DataStore.Remove(keyName);
        }

        public virtual void Clear()
        {
            DataStore.Clear();
        }
    }
}